"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stylable_1 = require("stylable");
const stylable_2 = require("stylable");
const completion_types_1 = require("./completion-types");
const postcss_ast_utils_1 = require("./utils/postcss-ast-utils");
const path = require("path");
const provider_1 = require("./provider");
const pvp = require('postcss-value-parser');
const uri_utils_1 = require("./utils/uri-utils");
const lodash_1 = require("lodash");
class ProviderPosition {
    constructor(line, character) {
        this.line = line;
        this.character = character;
    }
}
exports.ProviderPosition = ProviderPosition;
class ProviderRange {
    constructor(start, end) {
        this.start = start;
        this.end = end;
    }
}
exports.ProviderRange = ProviderRange;
class ProviderLocation {
    constructor(uri, range) {
        this.uri = uri;
        this.range = range;
    }
}
exports.ProviderLocation = ProviderLocation;
const cssPseudoClasses = [
    'active',
    'any',
    'checked',
    'default',
    'dir()',
    'disabled',
    'empty',
    'enabled',
    'first',
    'first-child',
    'first-of-type',
    'fullscreen',
    'focus',
    'hover',
    'indeterminate',
    'in-range',
    'invalid',
    'lang()',
    'last-child',
    'last-of-type',
    'left',
    'link',
    'not()',
    'nth-child()',
    'nth-last-child()',
    'nth-last-of-type()',
    'nth-of-type()',
    'only-child',
    'only-of-type',
    'optional',
    'out-of-range',
    'read-only',
    'read-write',
    'required',
    'right',
    'root',
    'scope',
    'target',
    'valid',
    'visited',
];
// const cssPseudoElements = [
//     '::after',
//     '::before',
//     '::cue',
//     '::first-letter',
//     '::first-line',
//     '::selection',
// ]
function createRange(startLine, startPos, endline, endPos) {
    return new ProviderRange(new ProviderPosition(startLine, startPos), new ProviderPosition(endline, endPos));
}
exports.createRange = createRange;
function createDirectiveRange(position, fullLineText, lineChunkAtCursor) {
    return new ProviderRange(new ProviderPosition(position.line, Math.max(0, position.character -
        (completion_types_1.topLevelDirectives.customSelector.startsWith(fullLineText)
            ? fullLineText.length
            : lineChunkAtCursor.length))), position);
}
const importDeclarations = ['default', 'named', 'from', 'theme'];
const simpleRulesetDeclarations = ['extends', 'states', 'variant', 'mixin'];
const topLevelDeclarations = ['root', 'namespace', 'vars', 'import', 'customSelector'];
//Providers
//Syntactic
// Inside :import ruleset, which is not inside media query
// If directive doesn't already exist
exports.ImportInternalDirectivesProvider = {
    provide({ parentSelector, isMediaQuery, fullLineText, position, lineChunkAtCursor }) {
        if (parentSelector && parentSelector.selector === ':import'
            && !isMediaQuery) {
            const res = [];
            importDeclarations.forEach(name => {
                if (parentSelector.nodes.every(n => postcss_ast_utils_1.isDeclaration(n) && completion_types_1.importDirectives[name] !== n.prop || postcss_ast_utils_1.isComment(n))
                    && completion_types_1.importDirectives[name].indexOf(fullLineText.trim()) === 0) {
                    res.push(completion_types_1.importInternalDirective(name, createDirectiveRange(position, fullLineText, lineChunkAtCursor)));
                }
            });
            return res;
        }
        else {
            return [];
        }
    }
};
// Inside ruleset, which is not :import or :vars
// Only inside simple selector, except -st-mixin
// If directive doesn't already exist
exports.RulesetInternalDirectivesProvider = {
    provide({ parentSelector, isMediaQuery, fullLineText, position, lineChunkAtCursor }) {
        let res = [];
        if (parentSelector && !(parentSelector.selector === ':import' || parentSelector.selector === ':vars')) {
            if (parentSelector.nodes.every(n => (postcss_ast_utils_1.isDeclaration(n) && completion_types_1.rulesetDirectives.mixin !== n.prop) || postcss_ast_utils_1.isComment(n))
                && completion_types_1.rulesetDirectives['mixin'].indexOf(fullLineText.trim()) === 0) {
                res.push(completion_types_1.rulesetInternalDirective('mixin', createDirectiveRange(position, fullLineText, lineChunkAtCursor)));
            }
            if (this.isSimpleSelector(parentSelector.selector) && !isMediaQuery) {
                simpleRulesetDeclarations.filter(d => d !== 'mixin').forEach(name => {
                    if (parentSelector.nodes.every(n => (postcss_ast_utils_1.isDeclaration(n) && completion_types_1.rulesetDirectives[name] !== n.prop) || postcss_ast_utils_1.isComment(n))
                        && completion_types_1.rulesetDirectives[name].indexOf(fullLineText.trim()) === 0) {
                        res.push(completion_types_1.rulesetInternalDirective(name, createDirectiveRange(position, fullLineText, lineChunkAtCursor)));
                    }
                });
            }
            return res;
        }
        else {
            return [];
        }
    },
    isSimpleSelector(sel) {
        return !!/^\s*\.?[\w-]*$/.test(sel); //Only a single class or element
    }
};
// Only top level
// :vars, @namespace may not repeat
exports.TopLevelDirectiveProvider = {
    provide({ parentSelector, isMediaQuery, fullLineText, position, lineChunkAtCursor, meta }) {
        if (!parentSelector) {
            if (!isMediaQuery) {
                return topLevelDeclarations
                    .filter(d => !/@namespace/.test(meta.ast.source.input.css) || (d !== 'namespace'))
                    .filter(d => completion_types_1.topLevelDirectives[d].indexOf(fullLineText.trim()) === 0)
                    .map(d => completion_types_1.topLevelDirective(d, createDirectiveRange(position, fullLineText, lineChunkAtCursor)));
            }
            else {
                return [completion_types_1.topLevelDirective('root', createDirectiveRange(position, fullLineText, lineChunkAtCursor))];
            }
        }
        else {
            return [];
        }
    },
};
// Inside ruleset, which is not :import
// RHS of declaration
// Declaration is not -st-directive (except -st-mixin)
// Not inside another value()
exports.ValueDirectiveProvider = {
    provide({ parentSelector, fullLineText, position }) {
        if (parentSelector && !provider_1.isDirective(fullLineText) && !this.isInsideValueDirective(fullLineText, position.character)
            && fullLineText.indexOf(':') !== -1) {
            const parsed = pvp(fullLineText.slice(fullLineText.indexOf(':') + 1)).nodes;
            const node = parsed[parsed.length - 1];
            if (node.type === 'div' || node.type === 'space'
                || node.type === 'function' && !node.unclosed
                || node.type === 'word' && 'value()'.startsWith(node.value)) {
                return [completion_types_1.valueDirective(new ProviderRange(new ProviderPosition(position.line, ~fullLineText.indexOf(',')
                        ? fullLineText.lastIndexOf(',') + 1
                        : fullLineText.indexOf(':') + 1), position))];
            }
            else {
                return [];
            }
        }
        else {
            return [];
        }
    },
    isInsideValueDirective(wholeLine, pos) {
        if (!/value\(/.test(wholeLine)) {
            return false;
        }
        let line = wholeLine.slice(0, pos).slice(wholeLine.lastIndexOf('value('));
        let stack = 0;
        for (let i = 0; i <= line.length; i++) {
            if (line[i] === '(') {
                stack += 1;
            }
            else if (line[i] === ')') {
                stack -= 1;
            }
        }
        return stack > 0;
    }
};
// Selector level
exports.GlobalCompletionProvider = {
    provide({ parentSelector, fullLineText, position, lineChunkAtCursor }) {
        if (!parentSelector && !lineChunkAtCursor.endsWith('::')) {
            let offset = 0;
            if (fullLineText.lastIndexOf(':') !== -1) {
                if (':global()'.startsWith(lineChunkAtCursor.slice(lineChunkAtCursor.lastIndexOf(':')))) {
                    offset = lineChunkAtCursor.slice(lineChunkAtCursor.lastIndexOf(':')).length;
                }
            }
            return [completion_types_1.globalCompletion(new ProviderRange(new ProviderPosition(position.line, position.character - offset), position))];
        }
        else {
            return [];
        }
    },
};
//Semantic
// Selector level
// Not after :, unless entire chunk is :
exports.SelectorCompletionProvider = {
    provide({ parentSelector, fullLineText, position, lineChunkAtCursor, meta, fakes, styl }) {
        if (!parentSelector && (lineChunkAtCursor === ':' || !lineChunkAtCursor.endsWith(':'))) {
            let comps = [];
            comps.push(...lodash_1.keys(meta.classes)
                .filter(c => c !== 'root' && fakes.findIndex(f => f.selector === '.' + c) === -1)
                .map(c => completion_types_1.classCompletion(c, (createDirectiveRange(position, fullLineText, lineChunkAtCursor)))));
            comps.push(...lodash_1.keys(meta.customSelectors)
                .map(c => completion_types_1.classCompletion(c, (createDirectiveRange(position, fullLineText, lineChunkAtCursor)), true)));
            let moreComps = meta.imports
                .filter(imp => imp.fromRelative.endsWith('st.css'))
                .reduce((acc, imp) => {
                if (acc.every(comp => comp.label !== imp.defaultExport)) {
                    acc.push(completion_types_1.classCompletion(imp.defaultExport, createDirectiveRange(position, fullLineText, lineChunkAtCursor), true));
                }
                ;
                lodash_1.keys(imp.named).forEach(exp => {
                    const res = styl.resolver.resolve(meta.mappedSymbols[exp]);
                    if (res && res._kind === 'css' && res.symbol && (res.symbol._kind === 'class' || res.symbol._kind === 'element') &&
                        (acc.every(comp => comp.label.replace('.', '') !== imp.named[exp]))) {
                        acc.push(completion_types_1.classCompletion(imp.named[exp], (createDirectiveRange(position, fullLineText, lineChunkAtCursor))));
                    }
                });
                return acc;
            }, comps);
            return moreComps.filter(c => c.label.startsWith(lineChunkAtCursor));
        }
        else {
            return [];
        }
    },
};
// Inside ruleset of simple selector, not :import or :vars
// RHS of -st-extends
exports.ExtendCompletionProvider = {
    provide({ lineChunkAtCursor, position, meta, styl }) {
        if (lineChunkAtCursor.startsWith(stylable_1.valueMapping.extends)) {
            let value = lineChunkAtCursor.slice((stylable_1.valueMapping.extends + ':').length);
            let spaces = value.search(/\S|$/);
            let str = value.slice(spaces);
            let comps = [[]];
            comps.push(...Object.keys(meta.classes).filter(s => s.startsWith(str)).map(s => [s, 'Local file']));
            meta.imports.forEach(i => { if (i.defaultExport && i.defaultExport.startsWith(str) && i.from.endsWith('st.css')) {
                comps.push([i.defaultExport, i.fromRelative]);
            } });
            meta.imports.forEach(i => comps.push(...lodash_1.keys(i.named)
                .filter(s => {
                const res = styl.resolver.resolve(meta.mappedSymbols[s]);
                return res && res._kind === 'css' && (res.symbol._kind === 'class' || res.symbol._kind === 'element');
            })
                .filter(s => s.startsWith(str))
                .map(s => [s, i.fromRelative])));
            return comps.slice(1).map(c => completion_types_1.extendCompletion(c[0], c[1], new ProviderRange(new ProviderPosition(position.line, position.character - str.length), position)));
        }
        else {
            return [];
        }
    },
};
// Inside ruleset, which is not :import or :vars
// RHS of -st-extends
exports.CssMixinCompletionProvider = {
    provide({ lineChunkAtCursor, meta, position, fullLineText }) {
        if (lineChunkAtCursor.startsWith(stylable_1.valueMapping.mixin + ':')) {
            const { names, lastName } = provider_1.getExistingNames(fullLineText, position);
            return Object.keys(meta.mappedSymbols)
                .filter(ms => ((meta.mappedSymbols[ms]._kind === 'import' && meta.mappedSymbols[ms].import.fromRelative.endsWith('st.css')) || meta.mappedSymbols[ms]._kind === 'class'))
                .filter(ms => ms.startsWith(lastName))
                .filter(ms => names.indexOf(ms) === -1)
                .map(ms => {
                return completion_types_1.cssMixinCompletion(ms, new ProviderRange(new ProviderPosition(position.line, position.character - lastName.length), position), meta.mappedSymbols[ms]._kind === 'import' ? meta.mappedSymbols[ms].import.fromRelative : 'Local file');
            });
        }
        else {
            return [];
        }
    },
};
// Mixin completions
// Inside ruleset, which is not :import or :vars
// Only inside simple selector
// RHS of -st-mixin
// There is  a JS/TS import
exports.CodeMixinCompletionProvider = {
    provide({ parentSelector, meta, fullLineText, lineChunkAtCursor, position, fs, tsLangService, styl }) {
        if (meta.imports.some(imp => imp.fromRelative.endsWith('.ts') || imp.fromRelative.endsWith('.js')) &&
            !fullLineText.trim().startsWith(stylable_1.valueMapping.from) &&
            parentSelector && lineChunkAtCursor.startsWith(stylable_1.valueMapping.mixin + ':')) {
            if (fullLineText.lastIndexOf('(') > fullLineText.lastIndexOf(')')) {
                return [];
            }
            const { names, lastName } = provider_1.getExistingNames(fullLineText, position);
            return Object.keys(meta.mappedSymbols)
                .filter(ms => meta.mappedSymbols[ms]._kind === 'import')
                .filter(ms => ms.startsWith(lastName))
                .filter(ms => {
                const res = styl.resolver.resolve(meta.mappedSymbols[ms]);
                return res && res._kind === 'js';
            })
                .filter(ms => isMixin(ms, meta, fs, tsLangService))
                .map(ms => createCodeMixinCompletion(ms, lastName, position, meta));
        }
        else {
            return [];
        }
    },
};
// Inside ruleset, which is not :import
// RHS of any rule except -st-extends, -st-from
exports.FormatterCompletionProvider = {
    provide({ meta, fullLineText, parentSelector, lineChunkAtCursor, position, fs, tsLangService, styl }) {
        if (meta.imports.some(imp => imp.fromRelative.endsWith('.ts') || imp.fromRelative.endsWith('.js')) &&
            !fullLineText.trim().startsWith(stylable_1.valueMapping.from) && !fullLineText.trim().startsWith(stylable_1.valueMapping.extends) && !fullLineText.trim().startsWith(stylable_1.valueMapping.named) &&
            parentSelector && ~fullLineText.indexOf(':') && fullLineText.indexOf(':') < position.character &&
            !lineChunkAtCursor.startsWith(stylable_1.valueMapping.mixin + ':')) {
            const { names, lastName } = provider_1.getExistingNames(fullLineText, position);
            return Object.keys(meta.mappedSymbols)
                .filter(ms => (meta.mappedSymbols[ms]._kind === 'import'))
                .filter(ms => ms.startsWith(lastName))
                .filter(ms => {
                const res = styl.resolver.resolve(meta.mappedSymbols[ms]);
                return res && res._kind === 'js';
            })
                .filter(ms => !isMixin(ms, meta, fs, tsLangService))
                .map(ms => createCodeMixinCompletion(ms, lastName, position, meta));
        }
        else {
            return [];
        }
    },
};
// Inside :import
// RHS of -st-named
// import exists
exports.NamedCompletionProvider = {
    provide({ parentSelector, astAtCursor, styl, meta, position, fullLineText, src }) {
        const { isNamedValueLine, namedValues } = provider_1.getNamedValues(src, position.line);
        if (isNamedValueLine) {
            let importName = '';
            if (parentSelector && parentSelector.selector === ':import' && astAtCursor.nodes && astAtCursor.nodes.length) {
                importName = astAtCursor.nodes.find(n => n.prop === stylable_1.valueMapping.from).value.replace(/'|"/g, '');
            }
            else {
                return [];
            }
            let comps = [[]];
            if (importName.endsWith('.st.css')) {
                const resolvedImport = this.resolveImport(importName, styl, meta);
                if (resolvedImport) {
                    const { names, lastName } = provider_1.getExistingNames(fullLineText, position);
                    comps.push(...lodash_1.keys(resolvedImport.mappedSymbols)
                        .filter(ms => (resolvedImport.mappedSymbols[ms]._kind === 'class' || resolvedImport.mappedSymbols[ms]._kind === 'var') && ms !== 'root')
                        .filter(ms => ms.slice(0, -1).startsWith(lastName))
                        .filter(ms => !~namedValues.indexOf(ms))
                        .map(ms => [
                        ms,
                        path.relative(meta.source, resolvedImport.source).slice(1).replace(/\\/g, '/'),
                        resolvedImport.mappedSymbols[ms]._kind === 'var' ? resolvedImport.mappedSymbols[ms].text : 'Stylable class'
                    ]));
                    return comps.slice(1).map(c => completion_types_1.namedCompletion(c[0], new ProviderRange(new ProviderPosition(position.line, position.character - lastName.length), new ProviderPosition(position.line, position.character)), c[1], c[2]));
                }
            }
            else if (importName.endsWith('.js') || importName.endsWith('.ts')) {
            }
        }
        return [];
    },
    resolveImport(importName, styl, meta) {
        let resolvedImport = null;
        if (importName && importName.endsWith('.st.css'))
            try {
                resolvedImport = styl.fileProcessor.process(meta.imports.find(i => i.fromRelative === importName).from);
            }
            catch (e) { }
        return resolvedImport;
    },
};
exports.PseudoElementCompletionProvider = {
    provide({ parentSelector, resolved, resolvedElements, lastSelectoid, lineChunkAtCursor, meta, position }) {
        let comps = [];
        if (!parentSelector && resolved.length > 0) {
            const lastNode = resolvedElements[0][resolvedElements[0].length - 1];
            const states = lastNode.resolved.reduce((acc, cur) => {
                acc = acc.concat(lodash_1.keys(cur.symbol[stylable_1.valueMapping.states]));
                return acc;
            }, cssPseudoClasses);
            let filter = lastNode.resolved.length
                ? ~states.indexOf(lastSelectoid.replace(':', ''))
                    ? ''
                    : lastSelectoid.replace(':', '')
                : lastNode.name;
            const scope = filter
                ? resolvedElements[0][resolvedElements[0].length - 2]
                : lastNode;
            const colons = lineChunkAtCursor.match(/:*$/)[0].length;
            scope.resolved.forEach(res => {
                if (!res.symbol[stylable_1.valueMapping.root]) {
                    return;
                }
                comps = comps.concat(lodash_1.keys(res.meta.classes)
                    .concat(lodash_1.keys(res.meta.customSelectors).map(s => s.slice(':--'.length)))
                    .filter(e => e.startsWith(filter) && e !== 'root')
                    .map(c => {
                    let relPath = path.relative(path.dirname(meta.source), res.meta.source);
                    if (!relPath.startsWith('.')) {
                        relPath = './' + relPath;
                    }
                    return completion_types_1.pseudoElementCompletion(c, relPath, new ProviderRange(new ProviderPosition(position.line, position.character - (filter ? filter.length + 2 : colons)), new ProviderPosition(position.line, position.character)));
                }));
            });
            let otherScope;
            if (!filter && lineChunkAtCursor.split('::').length > 1 && lodash_1.last(lineChunkAtCursor.split('::')) === scope.name) {
                otherScope = resolvedElements[0][resolvedElements[0].length - 2];
                filter = scope.name;
            }
            if (otherScope) {
                otherScope.resolved.forEach(res => {
                    if (!res.symbol[stylable_1.valueMapping.root]) {
                        return;
                    }
                    comps = comps.concat(lodash_1.keys(res.meta.classes)
                        .concat(lodash_1.keys(res.meta.customSelectors).map(s => s.slice(':--'.length)))
                        .filter(e => e.startsWith(filter) && e !== 'root')
                        .map(c => {
                        let relPath = path.relative(path.dirname(meta.source), res.meta.source);
                        if (!relPath.startsWith('.')) {
                            relPath = './' + relPath;
                        }
                        return completion_types_1.pseudoElementCompletion(c, relPath, new ProviderRange(new ProviderPosition(position.line, position.character - (filter ? filter.length + 2 : colons)), new ProviderPosition(position.line, position.character)));
                    }));
                });
            }
        }
        return comps;
    },
};
exports.StateCompletionProvider = {
    provide({ parentSelector, lineChunkAtCursor, resolvedElements, target, lastSelectoid, meta, position }) {
        if (!parentSelector && !lineChunkAtCursor.endsWith('::')) {
            const lastNode = resolvedElements[0][resolvedElements[0].length - 1];
            const chunk = Array.isArray(target.focusChunk) ? lodash_1.last(target.focusChunk) : target.focusChunk;
            const chunkyStates = (chunk && chunk.states) ? chunk.states : [];
            const allStates = lastNode.resolved.reduce((acc, cur) => {
                acc.push(...lodash_1.keys(cur.symbol[stylable_1.valueMapping.states]));
                return acc;
            }, []);
            const newStates = lastNode.resolved.reduce((acc, cur) => {
                let relPath = path.relative(path.dirname(meta.source), cur.meta.source);
                if (!relPath.startsWith('.')) {
                    relPath = './' + relPath;
                }
                lodash_1.keys(cur.symbol[stylable_1.valueMapping.states]).forEach(k => {
                    if (!acc[k] &&
                        (k.slice(0, -1).startsWith(lastSelectoid.replace(':', '')) || //selectoid is a substring of current state
                            ~allStates.indexOf(lastSelectoid.replace(':', '')) //selectoid is a valid state TODO: selectoid is both
                        ) &&
                        (chunkyStates.every(cs => cs !== k))) {
                        acc[k] = meta.source === cur.meta.source ? 'Local file' : relPath;
                    }
                });
                return acc;
            }, {});
            let states = lodash_1.keys(newStates).map(k => [k, newStates[k]]);
            if (states.length === 0) {
                return [];
            }
            ;
            const lastState = lastSelectoid.replace(':', '');
            const realState = ~allStates.indexOf(lastState);
            return states.reduce((acc, st) => {
                acc.push(completion_types_1.stateCompletion(st[0], st[1], (new ProviderRange(new ProviderPosition(position.line, lastState
                    ? realState
                        ? position.character - (lineChunkAtCursor.endsWith(':') ? 1 : 0)
                        : position.character - (lastState.length + 1) - (lineChunkAtCursor.endsWith(':') ? 1 : 0)
                    : position.character - (lineChunkAtCursor.endsWith(':') ? 1 : 0)), position))));
                return acc;
            }, []);
        }
        else {
            return [];
        }
    },
};
exports.ValueCompletionProvider = {
    provide({ fullLineText, position, meta, styl }) {
        if (provider_1.isInValue(fullLineText, position)) {
            let inner = fullLineText.slice(0, fullLineText.indexOf(')', position.character) + 1).slice(fullLineText.slice(0, fullLineText.indexOf(')', position.character) + 1).lastIndexOf('(')).replace('(', '').replace(')', '').trim();
            let comps = [];
            meta.vars.forEach(v => {
                if (v.name.startsWith(inner) && !~fullLineText.slice(0, fullLineText.indexOf(':')).indexOf(v.name)) {
                    const value = stylable_2.evalDeclarationValue(styl.resolver, v.text, meta, v.node);
                    comps.push(completion_types_1.valueCompletion(v.name, 'Local variable', value, new ProviderRange(new ProviderPosition(position.line, position.character - inner.length), position)));
                }
            });
            const importVars = [];
            meta.imports.forEach(imp => {
                try {
                    styl.fileProcessor.process(imp.from).vars.forEach(v => importVars.push({ name: v.name, value: v.text, from: imp.fromRelative, node: v.node }));
                }
                catch (e) { }
            });
            importVars.forEach(v => {
                if (v.name.startsWith(inner) && meta.imports.some(imp => Object.keys(imp.named).some(key => key === v.name))) {
                    const value = stylable_2.evalDeclarationValue(styl.resolver, v.value, meta, v.node);
                    comps.push(completion_types_1.valueCompletion(v.name, v.from, value, new ProviderRange(new ProviderPosition(position.line, position.character - inner.length), position)));
                }
            });
            return comps;
        }
        else {
            return [];
        }
    },
};
function createCodeMixinCompletion(name, lastName, position, meta) {
    return completion_types_1.codeMixinCompletion(name, new ProviderRange(new ProviderPosition(position.line, position.character - lastName.length), position), meta.mappedSymbols[name].import.fromRelative);
}
function isMixin(name, meta, fs, tsLangService) {
    const importSymbol = meta.mappedSymbols[name];
    if (importSymbol.import.fromRelative.endsWith('.ts')) {
        const sig = provider_1.extractTsSignature(importSymbol.import.from, name, importSymbol.type === 'default', tsLangService);
        if (!sig) {
            return false;
        }
        let rtype = sig.declaration.type
            ? sig.declaration.type.typeName.getText()
            : "";
        return (/(\w+.)?stCssFrag/.test(rtype.trim()));
    }
    if (importSymbol.import.fromRelative.endsWith('.js')) {
        return (provider_1.extractJsModifierRetrunType(name, 0, fs.get(uri_utils_1.toVscodePath(importSymbol.import.from)).getText()) === 'stCssFrag');
    }
    return false;
}
//# sourceMappingURL=completion-providers.js.map