"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PostCss = require("postcss");
const pvp = require('postcss-value-parser');
const psp = require('postcss-selector-parser');
const stylable_1 = require("stylable");
const postcss_ast_utils_1 = require("./utils/postcss-ast-utils");
const completion_providers_1 = require("./completion-providers");
const selector_analyzer_1 = require("./utils/selector-analyzer");
const path = require("path");
const ts = require("typescript");
const uri_utils_1 = require("./utils/uri-utils");
const lodash_1 = require("lodash");
const uri_utils_2 = require("./utils/uri-utils");
class Provider {
    constructor(styl, tsLangService) {
        this.styl = styl;
        this.tsLangService = tsLangService;
        this.providers = [
            completion_providers_1.RulesetInternalDirectivesProvider,
            completion_providers_1.ImportInternalDirectivesProvider,
            completion_providers_1.TopLevelDirectiveProvider,
            completion_providers_1.ValueDirectiveProvider,
            completion_providers_1.GlobalCompletionProvider,
            completion_providers_1.SelectorCompletionProvider,
            completion_providers_1.ExtendCompletionProvider,
            completion_providers_1.CssMixinCompletionProvider,
            completion_providers_1.CodeMixinCompletionProvider,
            completion_providers_1.FormatterCompletionProvider,
            completion_providers_1.NamedCompletionProvider,
            completion_providers_1.StateCompletionProvider,
            completion_providers_1.PseudoElementCompletionProvider,
            completion_providers_1.ValueCompletionProvider,
        ];
    }
    provideCompletionItemsFromSrc(src, pos, fileName, fs) {
        let res = fixAndProcess(src, pos, fileName);
        const completions = [];
        try {
            let options = this.createProviderOptions(src, pos, res.processed.meta, res.processed.fakes, res.currentLine, res.cursorLineIndex, fs);
            this.providers.forEach(p => { completions.push(...p.provide(options)); });
        }
        catch (e) { }
        return Promise.resolve(this.dedupeComps(completions));
    }
    createProviderOptions(src, position, meta, fakeRules, fullLineText, cursorPosInLine, fs) {
        const transformer = new stylable_1.StylableTransformer({
            diagnostics: new stylable_1.Diagnostics(),
            fileProcessor: this.styl.fileProcessor,
            requireModule: () => { throw new Error('Not implemented, why are we here'); }
        });
        const path = postcss_ast_utils_1.pathFromPosition(meta.rawAst, { line: position.line + 1, character: position.character });
        const astAtCursor = path[path.length - 1];
        const parentAst = astAtCursor.parent ? astAtCursor.parent : undefined;
        const parentSelector = parentAst && postcss_ast_utils_1.isSelector(parentAst) && fakeRules.findIndex((f) => { return f.selector === parentAst.selector; }) === -1
            ? parentAst
            : astAtCursor && postcss_ast_utils_1.isSelector(astAtCursor) && fakeRules.findIndex((f) => { return f.selector === astAtCursor.selector; }) === -1
                ? astAtCursor
                : null;
        const { lineChunkAtCursor, fixedCharIndex } = getChunkAtCursor(fullLineText, cursorPosInLine);
        const ps = selector_analyzer_1.parseSelector(lineChunkAtCursor, fixedCharIndex);
        const chunkStrings = ps.selector.reduce((acc, s) => { return acc.concat(s.text); }, []);
        const currentSelector = ps.selector[0].classes[0] || ps.selector[0].customSelectors[0] || chunkStrings[0];
        const expandedLine = stylable_1.expandCustomSelectors(PostCss.rule({ selector: lineChunkAtCursor }), meta.customSelectors).split(' ').pop(); // TODO: replace with selector parser
        const resolvedElements = transformer.resolveSelectorElements(meta, expandedLine);
        let resolved = [];
        if (currentSelector && resolvedElements[0].length) {
            const clas = resolvedElements[0].find(e => e.type === 'class' || (e.type === 'element' && e.resolved.length > 1)); //TODO: better type parsing
            resolved = clas ? clas.resolved : [];
        }
        return {
            meta: meta,
            fs: fs,
            styl: this.styl,
            src: src,
            tsLangService: this.tsLangService,
            resolvedElements: resolvedElements,
            parentSelector: parentSelector,
            astAtCursor: astAtCursor,
            lineChunkAtCursor: lineChunkAtCursor,
            lastSelectoid: ps.lastSelector,
            fullLineText: fullLineText,
            position: position,
            resolved: resolved,
            currentSelector: currentSelector,
            target: ps.target,
            isMediaQuery: isInMediaQuery(path),
            fakes: fakeRules,
        };
    }
    dedupeComps(completions) {
        let uniqs = new Map();
        completions.forEach(comp => {
            if (!uniqs.has(comp.label)) {
                uniqs.set(comp.label, comp);
            }
        });
        let res = [];
        uniqs.forEach(v => res.push(v));
        return res;
    }
    getDefinitionLocation(src, position, filePath, fs) {
        if (!filePath.endsWith('.st.css')) {
            return Promise.resolve([]);
        }
        let res = fixAndProcess(src, position, filePath);
        let meta = res.processed.meta;
        if (!meta) {
            return Promise.resolve([]);
        }
        ;
        const parsed = pvp(res.currentLine).nodes;
        let val = findNode(parsed, position.character);
        while (val.nodes && val.nodes.length > 0) {
            if (findNode(val.nodes, position.character).sourceIndex >= 0) {
                val = findNode(val.nodes, position.character);
            }
            else {
                break;
            }
        }
        let word = val.value;
        const { lineChunkAtCursor, fixedCharIndex } = getChunkAtCursor(res.currentLine.slice(0, val.sourceIndex + val.value.length), position.character);
        const transformer = new stylable_1.StylableTransformer({
            diagnostics: new stylable_1.Diagnostics(),
            fileProcessor: this.styl.fileProcessor,
            requireModule: () => { throw new Error('Not implemented, why are we here'); }
        });
        const expandedLine = stylable_1.expandCustomSelectors(PostCss.rule({ selector: lineChunkAtCursor }), meta.customSelectors).split(' ').pop(); // TODO: replace with selector parser
        const resolvedElements = transformer.resolveSelectorElements(meta, expandedLine);
        let defs = [];
        const reso = resolvedElements[0][resolvedElements[0].length - 1].resolved.find(res => {
            return res.symbol.name === word.replace('.', '') || lodash_1.keys(res.symbol[stylable_1.valueMapping.states]).some(k => k === word);
        });
        if (reso) {
            meta = reso.meta;
        }
        let temp = null;
        if (lodash_1.keys(meta.mappedSymbols).find(sym => sym === word.replace('.', ''))) {
            const symb = meta.mappedSymbols[word.replace('.', '')];
            switch (symb._kind) {
                case 'class': {
                    defs.push(new ProviderLocation(meta.source, this.findWord(word, fs.get(meta.source).getText(), position)));
                    break;
                }
                case 'var': {
                    defs.push(new ProviderLocation(meta.source, this.findWord(word, fs.get(meta.source).getText(), position)));
                    break;
                }
                case 'import': {
                    const filePath = path.join(path.dirname(meta.source), symb.import.fromRelative);
                    const doc = fs.get(filePath);
                    if (doc.getText() !== '') {
                        defs.push(new ProviderLocation(filePath, this.findWord(word, doc.getText(), position)));
                    }
                    ;
                    break;
                }
            }
        }
        else if (lodash_1.values(meta.mappedSymbols).some(k => {
            if (k._kind === 'class' && lodash_1.keys(k[stylable_1.valueMapping.states]).some(key => key === word)) {
                temp = k;
                return true;
            }
            else {
                return false;
            }
        })) {
            defs.push(new ProviderLocation(meta.source, this.findWord(temp.name, fs.get(meta.source).getText(), position)));
        }
        else if (lodash_1.keys(meta.customSelectors).find(sym => sym === ':--' + word)) {
            defs.push(new ProviderLocation(meta.source, this.findWord(':--' + word, src, position)));
        }
        return Promise.resolve(defs.filter(def => !this.inDef(position, def)));
    }
    inDef(position, def) {
        return (position.line > def.range.start.line || (position.line === def.range.start.line && position.character >= def.range.start.character))
            && (position.line < def.range.end.line || (position.line === def.range.end.line && position.character <= def.range.end.character));
    }
    findWord(word, src, position) {
        let split = src.split('\n');
        let regex = '\\b' + '\\.?' + this.escapeRegExp(word.replace('.', '').replace(':--', '')) + '\\b';
        let lineIndex = split.findIndex(l => RegExp(regex).test(l));
        if (lineIndex === -1 || lineIndex === position.line) {
            lineIndex = split.findIndex(l => l.trim().indexOf(word) !== -1);
        }
        if (lineIndex === -1 || lineIndex === position.line) {
            return completion_providers_1.createRange(0, 0, 0, 0);
        }
        ;
        let line = split[lineIndex];
        const match = line.match(RegExp(regex));
        if (match) {
            return completion_providers_1.createRange(lineIndex, line.lastIndexOf(word), lineIndex, line.lastIndexOf(word) + word.length);
        }
        else {
            return completion_providers_1.createRange(0, 0, 0, 0);
        }
    }
    escapeRegExp(re) {
        return re.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
    }
    getSignatureHelp(src, pos, filePath, fs, paramInfo) {
        if (!filePath.endsWith('.st.css')) {
            return null;
        }
        const res = fixAndProcess(src, pos, filePath);
        const meta = res.processed.meta;
        if (!meta)
            return null;
        const split = src.split('\n');
        const line = split[pos.line];
        let value = '';
        if (line.slice(0, pos.character).trim().startsWith(stylable_1.valueMapping.mixin)) {
            value = line.slice(0, pos.character).trim().slice(stylable_1.valueMapping.mixin.length + 1).trim();
        }
        else if (line.slice(0, pos.character).trim().includes(':')) {
            value = line.slice(0, pos.character).trim().slice(line.slice(0, pos.character).trim().indexOf(':') + 1).trim();
        }
        const parsed = pvp(value);
        let mixin = '';
        const rev = parsed.nodes.reverse()[0];
        if (rev.type === 'function' && !!rev.unclosed) {
            mixin = rev.value;
        }
        else {
            return null;
        }
        ;
        let activeParam = parsed.nodes.reverse()[0].nodes.reduce((acc, cur) => { return (cur.type === 'div' ? acc + 1 : acc); }, 0);
        if (mixin === 'value') {
            return null;
        }
        if (meta.mappedSymbols[mixin].import.from.endsWith('.ts')) {
            return this.getSignatureForTsModifier(mixin, activeParam, meta.mappedSymbols[mixin].import.from, meta.mappedSymbols[mixin].type === 'default', paramInfo);
        }
        else if (meta.mappedSymbols[mixin].import.from.endsWith('.js')) {
            if (fs.getOpenedFiles().indexOf(uri_utils_1.toVscodePath(meta.mappedSymbols[mixin].import.from.slice(0, -3) + '.d.ts')) !== -1) {
                return this.getSignatureForTsModifier(mixin, activeParam, meta.mappedSymbols[mixin].import.from.slice(0, -3) + '.d.ts', meta.mappedSymbols[mixin].type === 'default', paramInfo);
            }
            else {
                return this.getSignatureForJsModifier(mixin, activeParam, fs.get(meta.mappedSymbols[mixin].import.from).getText(), paramInfo);
            }
        }
        else {
            return null;
        }
    }
    getSignatureForTsModifier(mixin, activeParam, filePath, isDefault, paramInfo) {
        let sig = extractTsSignature(filePath, mixin, isDefault, this.tsLangService);
        let ptypes = sig.parameters.map(p => {
            return p.name + ":" + p.valueDeclaration.type.getFullText();
        });
        let rtype = sig.declaration.type
            ? sig.declaration.type.typeName.getFullText()
            : "";
        let parameters = sig.parameters.map(pt => {
            let label = pt.name + ":" + pt.valueDeclaration.type.getFullText();
            return paramInfo.create(label);
        });
        let sigInfo = {
            label: mixin + '(' + ptypes.join(', ') + '): ' + rtype,
            parameters
        };
        return {
            activeParameter: activeParam,
            activeSignature: 0,
            signatures: [sigInfo]
        };
    }
    getSignatureForJsModifier(mixin, activeParam, fileSrc, paramInfo) {
        let lines = fileSrc.split('\n');
        let mixinLine = lines.findIndex(l => l.trim().startsWith('exports.' + mixin));
        let docStartLine = lines.slice(0, mixinLine).lastIndexOf(lines.slice(0, mixinLine).reverse().find(l => l.trim().startsWith('/**')));
        let docLines = lines.slice(docStartLine, mixinLine);
        let formattedLines = [];
        docLines.forEach(l => {
            if (l.trim().startsWith('*/')) {
                return;
            }
            if (l.trim().startsWith('/**') && !!l.trim().slice(3).trim()) {
                formattedLines.push(l.trim().slice(3).trim());
            }
            if (l.trim().startsWith('*')) {
                formattedLines.push(l.trim().slice(1).trim());
            }
        });
        const returnStart = formattedLines.findIndex(l => l.startsWith('@returns'));
        const returnEnd = formattedLines.slice(returnStart + 1).findIndex(l => l.startsWith('@')) === -1
            ? formattedLines.length - 1
            : formattedLines.slice(returnStart + 1).findIndex(l => l.startsWith('@')) + returnStart;
        const returnLines = formattedLines.slice(returnStart, returnEnd + 1);
        formattedLines.splice(returnStart, returnLines.length);
        const returnType = /@returns *{(\w+)}/.exec(returnLines[0])
            ? /@returns *{(\w+)}/.exec(returnLines[0])[1]
            : '';
        const summaryStart = formattedLines.findIndex(l => l.startsWith('@summary'));
        let summaryLines = [];
        if (summaryStart !== -1) {
            const summaryEnd = formattedLines.slice(summaryStart + 1).findIndex(l => l.startsWith('@')) === -1
                ? formattedLines.length - 1
                : formattedLines.slice(summaryStart + 1).findIndex(l => l.startsWith('@')) + summaryStart;
            summaryLines = formattedLines.slice(summaryStart, summaryEnd + 1);
            formattedLines.splice(summaryStart, summaryLines.length);
        }
        let params = [];
        while (formattedLines.find(l => l.startsWith('@param'))) {
            const paramStart = formattedLines.findIndex(l => l.startsWith('@param'));
            const paramEnd = formattedLines.slice(paramStart + 1).findIndex(l => l.startsWith('@')) === -1
                ? formattedLines.length - 1
                : formattedLines.slice(paramStart + 1).findIndex(l => l.startsWith('@')) + paramStart;
            const paramLines = formattedLines.slice(paramStart, paramEnd + 1);
            formattedLines.splice(paramStart, paramLines.length);
            if (/@param *{([ \w<>,'"|]*)} *(\w*)(.*)/.exec(paramLines[0])) {
                params.push([
                    /@param *{([ \w<>,'"|]*)} *(\w*)(.*)/.exec(paramLines[0])[1],
                    /@param *{([ \w<>,'"|]*)} *(\w*)(.*)/.exec(paramLines[0])[2],
                    /@param *{([ \w<>,'"|]*)} *(\w*)(.*)/.exec(paramLines[0])[3],
                ]);
            }
        }
        let descLines = [];
        if (formattedLines.find(l => l.startsWith('@description'))) {
            const descStart = formattedLines.findIndex(l => l.startsWith('@description'));
            const descEnd = formattedLines.slice(descStart + 1).findIndex(l => l.startsWith('@')) === -1
                ? formattedLines.length - 1
                : formattedLines.slice(descStart + 1).findIndex(l => l.startsWith('@')) + descStart;
            descLines = formattedLines.slice(descStart, descEnd + 1);
        }
        else if (formattedLines.findIndex(l => l.startsWith('@')) === -1) {
            descLines = formattedLines;
        }
        else {
            descLines = formattedLines.slice(0, formattedLines.findIndex(l => l.startsWith('@')) + 1);
        }
        if (descLines[0] && descLines[0].startsWith('@description')) {
            descLines[0] = descLines[0].slice(12).trim();
        }
        let parameters = params.map(p => paramInfo.create(p[1] + ': ' + p[0], p[2].trim()));
        let sigInfo = {
            label: mixin + '(' + parameters.map(p => p.label).join(', ') + '): ' + returnType,
            documentation: descLines.join('\n'),
            parameters
        };
        return {
            activeParameter: activeParam,
            activeSignature: 0,
            signatures: [sigInfo]
        };
    }
    getRefs(params, fs) {
        const doc = fs.get(params.textDocument.uri).getText();
        const pos = { line: params.position.line + 1, character: params.position.character + 1 };
        const { meta } = createMeta(doc, params.textDocument.uri);
        const node = lodash_1.last(postcss_ast_utils_1.pathFromPosition(meta.rawAst, pos));
        let inner;
        let word = '';
        let refs = [];
        if (postcss_ast_utils_1.isRoot(node)) {
            inner = (node.nodes || []).find(n => {
                return (n.source.start.line < pos.line || (n.source.start.line === pos.line && n.source.start.column <= pos.character))
                    &&
                        (n.source.end.line > pos.line || (n.source.end.line === pos.line && n.source.end.column >= pos.character));
            });
            if (inner && postcss_ast_utils_1.isSelector(inner)) {
                const relPos = { line: pos.line - inner.source.start.line + 1, character: pos.character - inner.source.start.column + 1 };
                const proc = psp();
                const parsed = proc.astSync(inner.selector);
                const wordNode = (parsed).nodes[0].nodes
                    .find(n => {
                    return (n.source.start.line < relPos.line || (n.source.start.line === relPos.line && n.source.start.column <= pos.character))
                        &&
                            (n.source.end.line > relPos.line || (n.source.end.line === relPos.line && n.source.end.column >= pos.character));
                });
                if (wordNode) {
                    word = wordNode.value;
                }
            }
        }
        else if (postcss_ast_utils_1.isContainer(node)) {
            inner = (node.nodes || []).find(n => {
                return postcss_ast_utils_1.isDeclaration(n) &&
                    (n.prop === stylable_1.valueMapping.mixin || n.prop === stylable_1.valueMapping.extends) && (n.source.start.line < pos.line || (n.source.start.line === pos.line && n.source.start.column <= pos.character))
                    && (n.source.end.line > pos.line || (n.source.end.line === pos.line && n.source.end.column >= pos.character));
            });
            if (inner) {
                const parsed = pvp(inner.value);
                const relPos = inner.source.start.line === pos.line
                    ? pos.character - (inner.source.start.column + inner.prop.length + (inner.raws.between ? inner.raws.between.length : 0))
                    : pos.character - 1 + inner.value.split('\n').slice(0, pos.line - inner.source.start.line).reduce((acc, cur) => { acc += (cur.length + 1); return acc; }, 0);
                let val = findNode(parsed.nodes, relPos);
                if (val.type !== 'word') {
                    val = findNode(parsed.nodes, relPos - 1);
                }
                if (val) {
                    word = val.value;
                }
            }
        }
        refs = this.findClassRefs(word, params.textDocument.uri, fs);
        return refs;
    }
    findClassRefs(word, uri, fs) {
        if (!word) {
            return [];
        }
        ;
        const refs = [];
        const src = fs.get(uri).getText();
        const { processed: { meta } } = fixAndProcess(src, new completion_providers_1.ProviderPosition(0, 0), uri_utils_2.fromVscodePath(uri));
        const filterRegex = RegExp('(\\.?' + word + ')(\\s|$|\\:)', 'g');
        const valueRegex = RegExp('(\\.?' + word + ')(\\s|$|\\:|,)', 'g');
        meta.rawAst.walkRules(filterRegex, (rule) => {
            let match;
            while ((match = valueRegex.exec(rule.selector)) !== null) {
                refs.push({
                    uri,
                    range: {
                        start: {
                            line: rule.source.start.line - 1,
                            character: rule.source.start.column + match.index
                        },
                        end: {
                            line: rule.source.start.line - 1,
                            character: rule.source.start.column + match.index + word.length
                        }
                    }
                });
            }
        });
        meta.rawAst.walkDecls(stylable_1.valueMapping.extends, (decl) => {
            if (decl.value === word) {
                refs.push({
                    uri,
                    range: {
                        start: {
                            line: decl.source.start.line - 1,
                            character: decl.source.start.column + stylable_1.valueMapping.extends.length + (decl.raws.between ? decl.raws.between.length : 0) - 1
                        },
                        end: {
                            line: decl.source.start.line - 1,
                            character: decl.source.start.column + stylable_1.valueMapping.extends.length + (decl.raws.between ? decl.raws.between.length : 0) + word.length - 1
                        }
                    }
                });
            }
        });
        meta.rawAst.walkDecls(stylable_1.valueMapping.mixin, (decl) => {
            const lines = decl.value.split('\n');
            lines.forEach((line, index) => {
                let match;
                while ((match = valueRegex.exec(line)) !== null) {
                    refs.push({
                        uri,
                        range: {
                            start: {
                                line: decl.source.start.line - 1 + index,
                                character: index
                                    ? match.index
                                    : decl.source.start.column + stylable_1.valueMapping.mixin.length + match.index + (decl.raws.between ? decl.raws.between.length : 0) - 1
                            },
                            end: {
                                line: decl.source.start.line - 1 + index,
                                character: word.length + (index
                                    ? match.index
                                    : decl.source.start.column + stylable_1.valueMapping.mixin.length + match.index + (decl.raws.between ? decl.raws.between.length : 0) - 1)
                            }
                        }
                    });
                }
            });
        });
        return refs;
    }
}
exports.default = Provider;
function isIllegalLine(line) {
    return /^\s*[-\.:]+\s*$/.test(line);
}
const lineEndsRegexp = /({|}|;)/;
function createMeta(src, path) {
    let meta;
    let fakes = [];
    try {
        let ast = stylable_1.safeParse(src, { from: uri_utils_2.fromVscodePath(path) });
        ast.nodes && ast.nodes.forEach((node) => {
            if (node.type === 'decl') {
                let r = PostCss.rule({ selector: node.prop + ':' + node.value });
                r.source = node.source;
                node.replaceWith(r);
                fakes.push(r);
            }
        });
        if (ast.raws.after && ast.raws.after.trim()) {
            let r = PostCss.rule({ selector: ast.raws.after.trim() });
            ast.append(r);
            fakes.push(r);
        }
        meta = stylable_1.process(ast);
    }
    catch (error) {
        return { meta: null, fakes: fakes };
    }
    return {
        meta: meta,
        fakes: fakes
    };
}
exports.createMeta = createMeta;
function fixAndProcess(src, position, filePath) {
    let cursorLineIndex = position.character;
    let lines = src.replace(/\r\n/g, '\n').split('\n');
    let currentLine = lines[position.line];
    let fixedSrc = src;
    if (currentLine.match(lineEndsRegexp)) {
        let currentLocation = 0;
        let splitLine = currentLine.split(lineEndsRegexp);
        for (var i = 0; i < splitLine.length; i += 2) {
            currentLocation += splitLine[i].length + 1;
            if (currentLocation >= position.character) {
                currentLine = splitLine[i];
                if (isIllegalLine(currentLine)) {
                    splitLine[i] = '\n';
                    lines.splice(position.line, 1, splitLine.join(''));
                    fixedSrc = lines.join('\n');
                }
                break;
            }
            else {
                cursorLineIndex -= splitLine[i].length + 1;
            }
        }
    }
    else if (isIllegalLine(currentLine)) {
        lines.splice(position.line, 1, "");
        fixedSrc = lines.join('\n');
    }
    let processed = createMeta(fixedSrc, filePath);
    return {
        processed: processed,
        currentLine: currentLine,
        cursorLineIndex: cursorLineIndex,
    };
}
exports.fixAndProcess = fixAndProcess;
class ProviderLocation {
    constructor(uri, range) {
        this.uri = uri;
        this.range = range;
    }
}
exports.ProviderLocation = ProviderLocation;
function extractTsSignature(filePath, mixin, isDefault, tsLangService) {
    tsLangService.setOpenedFiles([filePath]);
    const program = tsLangService.ts.getProgram();
    const tc = program.getTypeChecker();
    const sf = program.getSourceFile(filePath);
    if (!sf) {
        return undefined;
    }
    const mix = tc.getSymbolsInScope(sf, ts.SymbolFlags.Function).find(f => {
        if (isDefault) {
            return f.exportSymbol && f.exportSymbol.escapedName === 'default';
        }
        else {
            return f.exportSymbol && f.exportSymbol.escapedName === mixin;
        }
    });
    if (!mix) {
        return undefined;
    }
    return tc.getSignatureFromDeclaration(mix.declarations[0]);
}
exports.extractTsSignature = extractTsSignature;
function extractJsModifierRetrunType(mixin, activeParam, fileSrc) {
    let lines = fileSrc.split('\n');
    let mixinLine = lines.findIndex(l => l.trim().startsWith('exports.' + mixin));
    let docStartLine = lines.slice(0, mixinLine).lastIndexOf(lines.slice(0, mixinLine).reverse().find(l => l.trim().startsWith('/**')));
    let docLines = lines.slice(docStartLine, mixinLine);
    let formattedLines = [];
    docLines.forEach(l => {
        if (l.trim().startsWith('*/')) {
            return;
        }
        if (l.trim().startsWith('/**') && !!l.trim().slice(3).trim()) {
            formattedLines.push(l.trim().slice(3).trim());
        }
        if (l.trim().startsWith('*')) {
            formattedLines.push(l.trim().slice(1).trim());
        }
    });
    const returnStart = formattedLines.findIndex(l => l.startsWith('@returns'));
    const returnEnd = formattedLines.slice(returnStart + 1).findIndex(l => l.startsWith('@')) === -1
        ? formattedLines.length - 1
        : formattedLines.slice(returnStart + 1).findIndex(l => l.startsWith('@')) + returnStart;
    const returnLines = formattedLines.slice(returnStart, returnEnd + 1);
    formattedLines.splice(returnStart, returnLines.length);
    const returnType = /@returns *{(\w+)}/.exec(returnLines[0])
        ? /@returns *{(\w+)}/.exec(returnLines[0])[1]
        : '';
    return returnType;
}
exports.extractJsModifierRetrunType = extractJsModifierRetrunType;
function isInMediaQuery(path) { return path.some(n => n.type === 'atrule' && n.name === 'media'); }
;
function isDirective(line) { return lodash_1.keys(stylable_1.valueMapping).some(k => line.trim().startsWith(stylable_1.valueMapping[k])); }
exports.isDirective = isDirective;
;
function isNamedDirective(line) { return line.indexOf(stylable_1.valueMapping.named) !== -1; }
;
function isInValue(lineText, position) {
    let isInValue = false;
    if (/value\(/.test(lineText)) {
        let line = lineText.slice(0, position.character);
        let stack = 0;
        for (let i = 0; i <= line.length; i++) {
            if (line[i] === '(') {
                stack += 1;
            }
            else if (line[i] === ')') {
                stack -= 1;
            }
        }
        if (stack > 0) {
            isInValue = true;
        }
    }
    return isInValue;
}
exports.isInValue = isInValue;
function getChunkAtCursor(fullLineText, cursorPosInLine) {
    let fixedCharIndex = cursorPosInLine;
    let lineChunkAtCursor = fullLineText;
    while (lineChunkAtCursor.lastIndexOf(' ') >= cursorPosInLine) {
        lineChunkAtCursor = lineChunkAtCursor.slice(0, lineChunkAtCursor.lastIndexOf(' '));
    }
    if (!isDirective(lineChunkAtCursor) && lineChunkAtCursor.lastIndexOf(' ') > -1 && lineChunkAtCursor.lastIndexOf(' ') < cursorPosInLine) {
        fixedCharIndex -= (lineChunkAtCursor.lastIndexOf(' ') + 1);
        lineChunkAtCursor = lineChunkAtCursor.slice(lineChunkAtCursor.lastIndexOf(' '));
    }
    return { lineChunkAtCursor: lineChunkAtCursor.trim(), fixedCharIndex };
}
function getNamedValues(src, lineIndex) {
    let lines = src.split('\n');
    let isNamedValueLine = false;
    let namedValues = [];
    for (let i = lineIndex; i > 0; i--) {
        if (isDirective(lines[i]) && !isNamedDirective(lines[i])) {
            break;
        }
        else if (isNamedDirective(lines[i])) {
            isNamedValueLine = true;
            let valueStart = lines[i].indexOf(':') + 1;
            let value = lines[i].slice(valueStart);
            value.split(',').map(x => x.trim()).filter(x => x !== '').forEach(x => namedValues.push(x));
            break;
        }
        else {
            let valueStart = lines[i].indexOf(':') + 1;
            let value = lines[i].slice(valueStart);
            value.split(',').map(x => x.trim()).filter(x => x !== '').forEach(x => namedValues.push(x));
        }
    }
    return { isNamedValueLine, namedValues };
}
exports.getNamedValues = getNamedValues;
function getExistingNames(lineText, position) {
    const valueStart = lineText.indexOf(':') + 1;
    const value = lineText.slice(valueStart, position.character);
    const parsed = pvp(value.trim());
    const names = parsed.nodes.filter((n) => n.type === 'function' || n.type === 'word').map((n) => n.value);
    const rev = parsed.nodes.reverse();
    const lastName = (parsed.nodes.length && rev[0].type === 'word') ? rev[0].value : '';
    return { names, lastName };
}
exports.getExistingNames = getExistingNames;
function findNode(nodes, index) {
    return nodes
        .filter(n => n.sourceIndex <= index)
        .reduce((m, n) => { return (m.sourceIndex > n.sourceIndex) ? m : n; }, { sourceIndex: -1 });
}
//# sourceMappingURL=provider.js.map