'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const vscode_languageserver_1 = require("vscode-languageserver");
const provider_factory_1 = require("./provider-factory");
const completion_providers_1 = require("./completion-providers");
const diagnosis_1 = require("./diagnosis");
const VCL = require("vscode-css-languageservice");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const stylable_1 = require("stylable");
const uri_utils_1 = require("./utils/uri-utils");
const provider_1 = require("./provider");
const stylable_2 = require("stylable");
const lodash_1 = require("lodash");
class StylableLanguageService {
    constructor(connection, services, fs, docsDispatcher, notifications) {
        const provider = provider_factory_1.createProvider(services.styl, services.tsLanguageService);
        const processor = provider.styl.fileProcessor;
        const cssService = VCL.getCSSLanguageService();
        let base;
        let symbolMap;
        connection.onInitialize((params) => {
            base = params.rootUri;
            return {
                capabilities: {
                    textDocumentSync: 1,
                    completionProvider: {
                        triggerCharacters: ['.', '-', ':', '"', ',']
                    },
                    definitionProvider: true,
                    hoverProvider: true,
                    referencesProvider: true,
                    renameProvider: true,
                    colorProvider: true,
                    signatureHelpProvider: {
                        triggerCharacters: [
                            '(',
                            ','
                        ]
                    },
                }
            };
        });
        connection.onCompletion((params) => {
            if (!params.textDocument.uri.endsWith('.st.css') && !params.textDocument.uri.startsWith('untitled:')) {
                return Promise.resolve([]);
            }
            const cssCompsRaw = cssService.doComplete(fs.get(params.textDocument.uri), params.position, cssService.parseStylesheet(fs.get(params.textDocument.uri)));
            const cssComps = cssCompsRaw ? cssCompsRaw.items : [];
            const doc = fs.get(params.textDocument.uri).getText();
            const pos = params.position;
            return provider.provideCompletionItemsFromSrc(doc, { line: pos.line, character: pos.character }, params.textDocument.uri, fs)
                .then((res) => {
                return res.map((com) => {
                    let lspCompletion = vscode_languageserver_types_1.CompletionItem.create(com.label);
                    let ted = vscode_languageserver_types_1.TextEdit.replace(com.range ? com.range : new completion_providers_1.ProviderRange(new completion_providers_1.ProviderPosition(pos.line, Math.max(pos.character - 1, 0)), pos), typeof com.insertText === 'string' ? com.insertText : com.insertText.source);
                    lspCompletion.insertTextFormat = 2;
                    lspCompletion.detail = com.detail;
                    lspCompletion.textEdit = ted;
                    lspCompletion.sortText = com.sortText;
                    lspCompletion.filterText = typeof com.insertText === 'string' ? com.insertText : com.insertText.source;
                    if (com.additionalCompletions) {
                        lspCompletion.command = vscode_languageserver_types_1.Command.create("additional", "editor.action.triggerSuggest");
                    }
                    else if (com.triggerSignature) {
                        lspCompletion.command = vscode_languageserver_types_1.Command.create("additional", "editor.action.triggerParameterHints");
                    }
                    return lspCompletion;
                }).concat(cssComps);
            });
        });
        function diagnose(document) {
            let cssDiags = document.uri.endsWith('.css')
                ? cssService.doValidation(document, cssService.parseStylesheet(document))
                    .filter(diag => {
                    if (diag.code === 'emptyRules') {
                        return false;
                    }
                    if (diag.code === 'css-unknownatrule' && readDocRange(document, diag.range) === '@custom-selector') {
                        return false;
                    }
                    if (diag.code === 'css-lcurlyexpected' && readDocRange(document, vscode_languageserver_types_1.Range.create(vscode_languageserver_types_1.Position.create(diag.range.start.line, 0), diag.range.end)).startsWith('@custom-selector')) {
                        return false;
                    }
                    if (diag.code === 'unknownProperties') {
                        let prop = diag.message.match(/'(.*)'/)[1];
                        let src = fs.loadTextFileSync(document.uri);
                        let meta = provider_1.createMeta(src, document.uri).meta;
                        if (meta && Object.keys(meta.mappedSymbols).some(ms => ms === prop)) {
                            return false;
                        }
                    }
                    return true;
                })
                    .map(diag => {
                    diag.source = 'css';
                    return diag;
                })
                : [];
            let diagnostics = diagnosis_1.createDiagnosis(document, fs, processor).map(diag => { diag.source = 'stylable'; return diag; });
            connection.sendDiagnostics({ uri: document.uri, diagnostics: diagnostics.concat(cssDiags) });
        }
        docsDispatcher.onDidOpen(function (params) {
            diagnose(params.document);
        });
        docsDispatcher.onDidChangeContent(function (params) {
            diagnose(params.document);
        });
        connection.onDefinition((params) => {
            const doc = fs.loadTextFileSync(params.textDocument.uri);
            const pos = params.position;
            return provider.getDefinitionLocation(doc, { line: pos.line, character: pos.character }, uri_utils_1.fromVscodePath(params.textDocument.uri), fs)
                .then((res) => {
                return res.map(loc => vscode_languageserver_types_1.Location.create(uri_utils_1.toVscodePath(loc.uri), loc.range));
            });
        });
        connection.onHover((params) => {
            let hover = cssService.doHover(fs.get(params.textDocument.uri), params.position, cssService.parseStylesheet(fs.get(params.textDocument.uri)));
            return Promise.resolve(hover); //Because Hover may be null, but not of null type - vscode code isn't strict that way.
        });
        connection.onReferences((params) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cssRefs = cssService.findReferences(fs.get(params.textDocument.uri), params.position, cssService.parseStylesheet(fs.get(params.textDocument.uri)));
            const refs = provider.getRefs(params, fs);
            return refs.length ? dedupeRefs(refs) : dedupeRefs(cssRefs);
        }));
        connection.onRequest(notifications.colorRequest.type, params => {
            const document = fs.get(params.textDocument.uri);
            const src = document.getText();
            const res = provider_1.fixAndProcess(src, new completion_providers_1.ProviderPosition(0, 0), params.textDocument.uri);
            const meta = res.processed.meta;
            let colorComps = [];
            const lines = src.split('\n');
            lines.forEach((line, ind) => {
                let valueRegex = /value\(([\w-]+)\)/g;
                let regexResult;
                while ((regexResult = valueRegex.exec(line)) !== null) {
                    const result = regexResult[1];
                    const sym = meta.mappedSymbols[result];
                    if (sym && sym._kind === 'var') {
                        const doc = vscode_languageserver_1.TextDocument.create('', 'css', 0, '.gaga {border: ' + stylable_2.evalDeclarationValue(services.styl.resolver, sym.text, meta, sym.node) + '}');
                        const stylesheet = cssService.parseStylesheet(doc);
                        const colors = cssService.findDocumentColors(doc, stylesheet);
                        const color = colors.length ? colors[0].color : null;
                        if (color) {
                            const range = new completion_providers_1.ProviderRange(new completion_providers_1.ProviderPosition(ind, regexResult.index + regexResult[0].indexOf(regexResult[1]) - 'value('.length), new completion_providers_1.ProviderPosition(ind, regexResult.index + regexResult[0].indexOf(regexResult[1]) + result.length));
                            colorComps.push({ color, range });
                        }
                    }
                    else if (sym && sym._kind === 'import' && sym.type === 'named') {
                        const impMeta = processor.process(sym.import.from);
                        const doc = vscode_languageserver_1.TextDocument.create('', 'css', 0, '.gaga {border: ' + stylable_2.evalDeclarationValue(services.styl.resolver, 'value(' + sym.name + ')', impMeta, impMeta.vars.find(v => v.name === sym.name).node) + '}');
                        const stylesheet = cssService.parseStylesheet(doc);
                        const colors = cssService.findDocumentColors(doc, stylesheet);
                        const color = colors.length ? colors[0].color : null;
                        if (color) {
                            const range = new completion_providers_1.ProviderRange(new completion_providers_1.ProviderPosition(ind, regexResult.index + regexResult[0].indexOf(regexResult[1]) - 'value('.length), new completion_providers_1.ProviderPosition(ind, regexResult.index + regexResult[0].indexOf(regexResult[1]) + result.length));
                            colorComps.push({ color, range });
                        }
                    }
                }
            });
            meta.imports.forEach(imp => {
                const impMeta = processor.process(imp.from);
                const vars = impMeta.vars;
                vars.forEach(v => {
                    const doc = vscode_languageserver_1.TextDocument.create('', 'css', 0, '.gaga {border: ' + stylable_2.evalDeclarationValue(services.styl.resolver, v.text, impMeta, v.node) + '}');
                    const stylesheet = cssService.parseStylesheet(doc);
                    const colors = cssService.findDocumentColors(doc, stylesheet);
                    const color = colors.length ? colors[0].color : null;
                    if (color) {
                        meta.rawAst.walkDecls(stylable_1.valueMapping.named, (decl) => {
                            const lines = decl.value.split('\n');
                            const lineIndex = lines.findIndex(l => l.includes(v.name)); //replace with regex
                            if (lineIndex > -1 && lines[lineIndex].indexOf(v.name) > -1) {
                                let extraLines = 0;
                                let extraChars = 0;
                                if (decl.raws.between) {
                                    extraLines = decl.raws.between.split('\n').length - 1;
                                    extraChars = lodash_1.last(decl.raws.between.split('\n')).length;
                                }
                                const varStart = lineIndex //replace with value parser
                                    ? lines[lineIndex].indexOf(v.name) //replace with regex
                                    : extraLines
                                        ? lines[lineIndex].indexOf(v.name) + extraChars
                                        : lines[lineIndex].indexOf(v.name) + stylable_1.valueMapping.named.length + decl.source.start.column + extraChars - 1;
                                const range = new completion_providers_1.ProviderRange(new completion_providers_1.ProviderPosition(decl.source.start.line - 1 + lineIndex + extraLines, varStart), new completion_providers_1.ProviderPosition(decl.source.start.line - 1 + lineIndex + extraLines, v.name.length + varStart));
                                colorComps.push({ color, range });
                            }
                        });
                    }
                });
            });
            const stylesheet = cssService.parseStylesheet(document);
            colorComps.push(...cssService.findDocumentColors(document, stylesheet));
            return colorComps;
        });
        connection.onRequest(notifications.colorPresentationRequest.type, params => {
            const document = fs.get(params.textDocument.uri);
            const src = document.getText();
            const res = provider_1.fixAndProcess(src, new completion_providers_1.ProviderPosition(0, 0), params.textDocument.uri);
            const meta = res.processed.meta;
            const word = src.split('\n')[params.range.start.line].slice(params.range.start.character, params.range.end.character);
            if (word.startsWith('value(')) {
                return [];
            }
            ;
            const wordStart = new completion_providers_1.ProviderPosition(params.range.start.line + 1, params.range.start.character + 1);
            let noPicker = false;
            meta.rawAst.walkDecls(stylable_1.valueMapping.named, (node) => {
                if (((wordStart.line === node.source.start.line && wordStart.character >= node.source.start.column) || wordStart.line > node.source.start.line)
                    &&
                        ((wordStart.line === node.source.end.line && wordStart.character <= node.source.end.column) || wordStart.line < node.source.end.line)) {
                    noPicker = true;
                }
            });
            if (noPicker) {
                return [];
            }
            ;
            const stylesheet = cssService.parseStylesheet(document);
            const colors = cssService.getColorPresentations(document, stylesheet, params.color, params.range);
            return colors;
        });
        connection.onRenameRequest((params) => {
            let edit = { changes: {} };
            provider.getRefs({ context: { includeDeclaration: true }, position: params.position, textDocument: params.textDocument }, fs)
                .forEach(ref => {
                if (edit.changes[ref.uri]) {
                    edit.changes[ref.uri].push({ range: ref.range, newText: params.newName });
                }
                else {
                    edit.changes[ref.uri] = [{ range: ref.range, newText: params.newName }];
                }
            });
            return edit;
        });
        connection.onSignatureHelp((params) => {
            const doc = fs.loadTextFileSync(params.textDocument.uri);
            let sig = provider.getSignatureHelp(doc, params.position, params.textDocument.uri, fs, vscode_languageserver_types_1.ParameterInformation);
            return Promise.resolve(sig);
        });
        function readDocRange(doc, rng) {
            let lines = doc.getText().split('\n');
            return lines[rng.start.line].slice(rng.start.character, rng.end.character);
        }
        function dedupeRefs(refs) {
            let res = [];
            refs.forEach(ref => {
                if (!res.find(r => r.range.start.line === ref.range.start.line && r.range.start.character === ref.range.start.character && r.uri === ref.uri)) {
                    res.push(ref);
                }
            });
            return res;
        }
    }
}
exports.StylableLanguageService = StylableLanguageService;
//# sourceMappingURL=service.js.map